function [lambdaMax,msdTrue,lambda,msdTrue2,lambda2,msdEin2] = findLambdaMax(msd,lambda,nParticles,dim,rho,isPer)
% Find lambdaMax by using Frenkel's cutoff argument in Understanding
% Molecular Simulation Sec. 10.3. Takes msd, lambda, nParticles, dimension,
% number density, and a boolean stating whether this is a periodic crystal
% vs. the einstein crystal. Outputs lambdaMax, lambda, a functional lambda2
% which is interpolated, the msd of the einstein crystal 'msdEin2', and the
% modified einstein crystal msd 'msdTrue' and its interpolated values
% 'msdTrue2'.


% lambda are the actual lambda used, whereas lambda2 is interpolated to
% many points and used for plotting functionals
lambda2 = logspace(log10(min(lambda)),log10(max(lambda)),500);

% Choose a fractional cutoff to distinguish between the measured msd
% and the analytic one. We choose lambdaMax such that this difference
% is less than the cutoff
fractionalCutoff = 0.01;

cryRho = [1,2*sqrt(3)/3,sqrt(2),2,2*sqrt(2),8*sqrt(3)/3,8,16,16*sqrt(2),40];
nNeighbors = [1,6,12,24,40,72,126,240,272,372];

% rhoc is the closest pack crystal density. nn is the number of contacts
rhoc = cryRho(dim);
nn = nNeighbors(dim);

% a is the distance between particles, sigma is the diameter
sigma = 1;
a = sigma*(rho/rhoc)^(-1/dim);

if isPer == 1
    msdEin = (dim/2) ./ lambda;
    msdEin2 = (dim/2) ./ lambda2;
else
    msdEin = (dim/2 * (nParticles - 1)/nParticles ./ lambda);
    msdEin2 = (dim/2 * (nParticles - 1)/nParticles ./ lambda2);
end

% pnn from and msdTrue from Frenkel's Understanding Molecular Simulation
% (2003), Eqs. 10.3.32 and 10.3.31 respectively 
pnn = erf(sqrt(lambda/2)*(sigma+a))/2 + erf(sqrt(lambda/2)*(sigma-a))/2 - (exp(-lambda*(sigma-a)^2/2) - exp(-lambda*(sigma+a)^2/2))./(sqrt(2*pi*lambda)*a);
pnn2 = erf(sqrt(lambda2/2)*(sigma+a))/2 + erf(sqrt(lambda2/2)*(sigma-a))/2 - (exp(-lambda2*(sigma-a)^2/2) - exp(-lambda2*(sigma+a)^2/2))./(sqrt(2*pi*lambda2)*a);

msdTrue = msdEin - nn/2*((sigma*a-sigma^2-1./lambda).*exp(-lambda*(a-sigma)^2/2) + (sigma*a+sigma^2-1./lambda).*exp(-lambda*(a+sigma)^2/2))./(2*a.*sqrt(2*pi*lambda).*(1-pnn));
msdTrue2 = msdEin2 - nn/2*((sigma*a-sigma^2-1./lambda2).*exp(-lambda2*(a-sigma)^2/2) + (sigma*a+sigma^2-1./lambda2).*exp(-lambda2*(a+sigma)^2/2))./(2*a.*sqrt(2*pi*lambda2).*(1-pnn2));

% Look for difference between the perfect einstein crystal msd and the one
% measured. Take the first value above the cutoff.
x = abs(msd - msdTrue)./msd;
below = find(x>fractionalCutoff);

% If lambdaMax is too low, manually raise it. This can be adjusted if
% needed.

if numel(below) == 0 || below(end) == numel(lambda)
    lambdaMax = lambda(end);
else
    lambdaMax = 2*lambda(below(end)+1);
end

if lambdaMax < 1400
    lambdaMax = 1400;
end

lambda = lambda';

if isPer == 1
    lambdaMax = 1e4;
end
if dim == 10
    lambdaMax = 2000;
end

end