function [rhoF, rhoM, pco, muco] = liquidCrystalCoexistencePoint(dim,nParticles,FRef,rhoRef)
% This function calculates the coexistence point for dimension 'dim',
% nParticles, and a given free energy 'FRef', and number density 'rhoRef'.
% In practice, we first find the infinite N extrapolation of FRef, and then
% set nParticles large (>1e5) in order to calculate the coexistence
% conditions. Outputs are the freezing density 'rhoF', the melting density
% 'rhoM', the coexistence pressure 'pCo', and the coexistence chemical
% potential 'muco'.

toDelete = 0;
unitless = 0;

lw = 2.5;

dimColor = zeros(8,3);
dimColor(:,1) = linspace(0,1,8);
dimColor(:,3) = linspace(1,0,8);

% Create interpolation values
if dim==3
    rhoL = linspace(0,1.05,100);
    rhoS = linspace(0.9,1.1,100);
    rhoS = linspace(0.9,1.272792206135794,100);
elseif dim==4
    rhoL = linspace(0,1.1,100);
    rhoS = linspace(0.97,1.5,100);
elseif dim==5
    rhoL = linspace(0,1.4,100);
    rhoS = linspace(1.3,1.7,100);
elseif dim==6
    rhoL = linspace(0,2,100);
    rhoS = linspace(1.2,3,100);
elseif dim ==7
    rhoL = linspace(0,2.5,100);
    rhoS = linspace(1.65,3,100);
elseif dim==8
    rhoL = linspace(0,3,100);
    rhoS = linspace(2.7,4,100);
elseif dim==9
    rhoL = linspace(0,9,100);
    rhoS = linspace(2.4,9,100);
elseif dim==10
    rhoL = linspace(0,12,100);
    rhoS = linspace(5,10,100);
end

FL = 0*rhoL;
muL = 0*rhoL;

for i = 1:numel(rhoL)
    %Find points (gP) and weights (gW) for Gauss quadrature
    [gP,gW] = lgwt(30,0,rhoL(i));
    f = 0*gP;
    
    %Apply function to gP. Here, we're integrating pressure/rho^2 - 1/rho
    for j = 1:numel(gP)
        f(j) = zMinusOneOverRhoF(gP(j),dim,nParticles);
    end
    
    %Calculate Free Energy by taking integral, and adding ideal gas
    %contribution
    FL(i) = sum(f.*gW) + log(rhoL(i)) - 1 + log(2*pi*nParticles)/(2*nParticles);
%     FL(i) = sum(f.*gW);

    %Calculate chemical potential
    muL(i) = FL(i) + (zMinusOneOverRhoF(rhoL(i),dim,nParticles).*rhoL(i))+1;
%    muL(i) = FL(i) + pF(rhoL(i),dim)./rhoL(i);
end

FS = 0*rhoS;
muS = 0*rhoS;

for i = 1:numel(rhoS)
    %Find points (gP) and weights (gW) for Gauss quadrature
    [gP,gW] = lgwt(30,rhoRef,rhoS(i));
    f = 0*gP;
    
    %Apply function to gP. Here, we're integrating pressure/rho^2
    for j = 1:numel(gP)
        f(j) = pS(gP(j),dim,nParticles)/(gP(j)^2);
    end
    
    %Calculate Free Energy by taking integral, and adding ideal gas
    %contribution
    FS(i) = sum(f.*gW) + FRef + log(rhoRef) - 1 + log(2*pi*nParticles)/(2*nParticles);% + (dim-1)/2 * log(nParticles)/nParticles;
    
    %Calculate chemical potential
    muS(i) = FS(i) + pS(rhoS(i),dim,nParticles)./rhoS(i);
end

%Coexistence point is when pressure and chemical potential are equal, which
%is the point at which these lines intersect
figure(8)
if toDelete
    clf; hold all
end
plot((zMinusOneOverRhoF(rhoL,dim,nParticles).*rhoL+1).*rhoL/dim,muL/dim,'-','Color',dimColor(dim-2,:),'Linewidth',lw)
% plot(pF(rhoL,dim),muL2,'--r')
if unitless
    h = plot(pS(rhoS,dim,nParticles)./rhoS/dim,muS/dim,'--','Color',dimColor(dim-2,:),'linewidth',lw);
else
    h = plot(pS(rhoS,dim,nParticles)/dim,muS/dim,'--','Color',dimColor(dim-2,:),'linewidth',lw);
end
set(get(get(h,'Annotation'),'LegendInformation'),'IconDisplayStyle','off');
ylabel('\mu/d')
if unitless
    xlabel('p/d')
else
    xlabel('P/d')
end


xl = (zMinusOneOverRhoF(rhoL,dim,nParticles).*rhoL+1).*rhoL;
yl = muL;
xs = pS(rhoS,dim,nParticles);
ys = muS;
coex = InterX([xs xl; ys yl]);
pco = coex(1);
muco = coex(2);
[res,~] = fit(pS(rhoS,dim,nParticles)',rhoS','spline');
rhoM = feval(res,pco);
[res,~] = fit(xl(2:end)',rhoL(2:end)','spline');
rhoF = feval(res,pco);

end

% pressure of liquid state from Clisby and McCoy, J. Stat. Phys. (2006)
function [P] = pF(rho,dim)
b2 = 0.5*hyperBallVolume(dim);
b = [1, 0.625, 0.2869495, 0.1102521, 0.0388819891, 0.0130235491, 0.004183211, 0.001309413, 0.000403515;
    1, 0.506340, 0.15184606, 0.035704117, 0.007735916, 0.001430319, 0.000288818, 0.000044122, 0.000011331;
    1, 0.414063, 0.0759724807, 0.012955113, 0.000981514, 0.000416219, -0.000112020, 0.000074726, -0.000049248
    1, 0.340941, 0.03336314, 0.007523111, -0.001738513, 0.001306618, -0.000895030, 0.000667345, -0.00052516
    1, 0.282227, 0.00986494662, 0.007072410 -0.003512111, 0.002538616, -0.001993728, 0.001686941, -0.00151414
    1, 0.234614, -0.00255768, 0.0074309293, -0.004516411, 0.003414915, -0.002862426, 0.002596938, -0.00251113];
P = rho;
for i = 2:10
    P = P + rho.^i * b2^(i-1) * b(dim-2,i-1);
end
if dim == 5
    P = 8.982*rho.^4 - 11.71*rho.^3 + 12.42*rho.^2 - 0.9876*rho;
end
if dim == 6
    P = 3.223*rho.^4 - 4.177*rho.^3 + 8.319*rho.^2 -0.5575*rho;
end
end

% Virial from pade approximants
function [Z] = zMinusOneOverRhoF(rho,dim,nParticles)
    if dim == 3
        Z = ((1 + 1.03544*rho + 0.887782*rho.^2 + 0.314355*rho.^3 + 0.062765*rho.^4)./(1 - 1.05896*rho + 0.364099*rho.^2 - 0.18124*rho.^3 + 0.114402*rho.^4 - 0.0230037*rho.^5) - 1)./rho;
        x = (pi*rho/6)./(1-pi*rho/6);
        Z = ((1 + 4.*x + 6*x.^2 + 2.3647684*x.^3 - 0.8698551*x.^4 + 1.1062803*x.^5 - 1.095049*x.^6 + 0.637614*x.^7 - 0.2279397*x.^10 + 0.1098948*x.^14 - 0.00906797*x.^22) - 1)./rho;
        Z = ((1 + 4.*x + 6*x.^2 + 2.3647684*x.^3 - 0.8698551*x.^4 + 1.1062803*x.^5 - 1.1014221*x.^6 + 0.66605866*x.^7 - 0.03633431*x.^8 - 0.20965164*x.^10 + 0.10555569*x.^14 - 0.00872380*x.^22) - 1)./rho;
    elseif dim ==4
        Z = ((1 + 2.0618*rho + 1.5498*rho.^2 + 0.1215*rho.^3 - 0.3273*rho.^4)./(1 - 0.4056*rho - 0.5320*rho.^2 + 0.4036*rho.^3 - 0.0813*rho.^4 - 0.0007*rho.^5)-1)./rho;
    elseif dim ==5
        Z = ((1 + 5.6285*rho + 11.4725*rho.^2 + 10.5480*rho.^3 + 3.6629*rho.^4)./(1 + 2.9966*rho + 0.7176*rho.^2 - 1.3205*rho.^3 + 0.3080*rho.^4 - 0.0039*rho.^5)-1)./rho;
    elseif dim == 6
        Z = ((1 + 5.6358*rho + 11.648*rho.^2 + 10.539*rho.^3 + 3.4170*rho.^4)./(1 + 3.0520*rho + 1.4857*rho.^2 - 0.8228*rho.^3 + 0.0694*rho.^4 + 0.0154*rho.^5)-1)./rho;
    elseif dim ==7
        Z = ((1 + 5.8810*rho + 12.461*rho.^2 + 11.291*rho.^3 + 3.5776*rho.^4)./(1 + 3.5187*rho + 2.5736*rho.^2 - 0.4615*rho.^3 - 0.0638*rho.^4 + 0.0261*rho.^5)-1)./rho;
    elseif dim == 8
        Z = ((1 + 5.4951*rho + 10.767*rho.^2 + 8.8764*rho.^3 + 2.5049*rho.^4)./(1 + 3.4657*rho + 2.7671*rho.^2 - 0.0664*rho.^3 - 0.0859*rho.^4 + 0.0163*rho.^5)-1)./rho;
    elseif dim == 9
        Z = ((1 + 4.9001*rho + 8.5158*rho.^2 + 6.1099*rho.^3 + 1.5317*rho.^4)./(1 + 3.2509*rho + 2.6220*rho.^2 + 0.1687*rho.^3 - 0.0721*rho.^4 + 0.0097*rho.^5)-1)./rho;
%         Z = (padeApproxHSFluid(rho,dim,1)-1)./rho;
    elseif dim == 10
        Z = (((1.0*(- 4.0009598825721592090871814293362e+34*rho.^15 - 2.909930917092161554334591896253e+35*rho.^14 - 7.408092696815907312543807478394e+35*rho.^13 - 1.0235804823814123024220711639607e+36*rho.^12 - 9.8634889958967913735268496332358e+35*rho.^11 - 6.6799994378717669726882992432227e+35*rho.^10 - 2.1261306270421601306502965917622e+35*rho.^9 + 4.0278790019462764153919181680184e+34*rho.^8 + 6.9919635708031810105213547213699e+34*rho.^7 + 5.3718683654084246876214367918876e+34*rho.^6 + 2.512124948395554089407271589192e+34*rho.^5 + 2.312522653718797085497444610243e+33*rho.^4 - 195433680370035859437922843002300.0*rho.^3 + 1.7895417484346576073242559012799e+33*rho.^2 + 1.1020883971442893103826929762115e+33*rho + 195282308424392716238113037955722.0))./(831853892587849018979014822200266.0*rho.^15 - 3.7640087035452653846515838679713e+33*rho.^14 - 1.5048003417579781148030027285338e+35*rho.^13 - 3.7798956893939028723089369895412e+35*rho.^12 - 4.2747684338794181865775626204948e+35*rho.^11 - 3.9422124956009935356660769450825e+35*rho.^10 - 2.1313976644010945420905514415248e+35*rho.^9 - 5.8214969501740510130013850702754e+33*rho.^8 + 3.0959371526389611559521595896599e+34*rho.^7 + 2.6322779294868143845561416271807e+34*rho.^6 + 2.0680069261250634758356320172696e+34*rho.^5 + 3.7448959063003138569060984215464e+33*rho.^4 - 1.2466348065041457027832039890061e+33*rho.^3 + 649802058383635521767846816016700.0*rho.^2 + 853087436860227804258409947091711.0*rho + 195282308424392716238113037955722.0))-1)./rho;
%         Z = (((1.0*(- 2.1958682921665764255570891466465e+35*rho.^14 - 9.7300700833397217277101279477685e+35*rho.^13 - 1.6192644410990425321592120839081e+36*rho.^12 - 1.3580674240138153726976393478921e+36*rho.^11 - 6.715412416167645942945592068471e+35*rho.^10 - 1.7012809565303989917656403059755e+35*rho.^9 + 6.6267291988432821663713954804954e+34*rho.^8 + 9.665857355583752525965086886769e+34*rho.^7 + 6.23309718412462342558470951708e+34*rho.^6 + 2.3206337153082675618090579523882e+34*rho.^5 + 1.2682826307794690397937779784831e+33*rho.^4 + 1.4181276374421787226820061083825e+33*rho.^3 + 2.2695533412546230486542222706643e+33*rho.^2 + 1.0243923473276535264166543178052e+33*rho + 257456279736952212396275173290944.0))./(312558235292578623267382507079399.0*rho.^16 - 2.2732616646106479746914624881598e+33*rho.^15 + 1.5311864618849515531956314638794e+34*rho.^14 - 7.7697582109438637606574770254451e+34*rho.^13 - 5.5171724891185899185078244084737e+35*rho.^12 - 7.4780452413270306445510431265902e+35*rho.^11 - 4.4171227540881048311088583524733e+35*rho.^10 - 1.7920095228038918546825736046521e+35*rho.^9 - 2.3434775938622876017374198088523e+33*rho.^8 + 4.693916759247951047271229109667e+34*rho.^7 + 3.4558918806824004049949195596677e+34*rho.^6 + 2.144894502772541259383979544156e+34*rho.^5 + 1.4857142697796890333763006943639e+33*rho.^4 - 436038162305882236342070386208700.0*rho.^3 + 1.3134166330157677006851295280398e+33*rho.^2 + 696114474114764580786510666175244.0*rho + 257456279736952212396275173290944.0))-1)./rho;
    end
 
    if dim == 8
        Z = 0.1153*rho.^2 + 0.8625*rho + 2.057;
    end
    
    if dim == 9
        Z = 0.02572*rho.^2 + 0.4777*rho + 1.663;
    end
end

% Pressure of the solid
function[P] = pS(rho,dim,nParticles)
cry = 2^(dim/2 - 1);
if dim == 6
    cry = pi^3/(48 * sqrt(3)) / hyperBallVolume(6) * 2^6;
elseif dim == 7
    cry = 8;
elseif dim == 8
    cry = pi^4/(2^4 * 4*3*2) / hyperBallVolume(8) * 2^8;
elseif dim == 9
    cry = cry*2;
elseif dim == 10
    cry = 40;
end
x = rho/cry;
if dim == 3
    P = (3./(1-x) - 0.5921.*(x - 0.7072)./(x - 0.601)).*rho;
    P = (3./(1-x) - 0.5864.*(x - 0.7085)./(x - 0.6055)).*rho;
%     P = rho./(1-x.^(1./dim)) + rho.*(13.56 - 30.3*x + 17.87*x.^2);
%    P = rho./(1-x.^(1./dim)) + rho.*(1.871 - 1.3*x);
elseif dim == 4
    P = (4./(1-x) - 1.124.*(x - 0.485)./(x - 0.4122)).*rho;
    P = rho./(1-x.^(1./dim)) + rho.*(4.003 - 7.761*x + 4.485*x.^2);
elseif dim == 5
    P = (5./(1-x) - 1.507.*(x - 0.412)./(x - 0.3841)).*rho;
    P = rho./(1-x.^(1./dim)) + rho.*(3.637 - 7.311*x + 4.564*x.^2);
%     P = 26.13.*rho.^2 - 57.57.*rho + 41.54;
elseif dim ==6
%     P = (6./(1-x) - 1.986.*(x - 0.3536)./(x - 0.3506)).*rho;
%     P = 12.53*rho.^2 - 27.41*rho + 24.81;
    P = rho./(1-x.^(1./dim)) + rho.*(2.4 - 3.1*x);
    P = rho./(1-x.^(1./dim)) + rho.*(3.016 - 7.05*x + 5.485*x.^2);
elseif dim ==7
%     P = (7./(1-x) - 2.593.*(x - 0.2)./(x - 0.1991)).*rho;
    P = rho./(1-x.^(1./dim)) + rho.*(2.669 - 6.984*x + 6.311*x.^2);
%     P = rho./(1-x.^(1./dim)) + rho.*(2.015 - 3.418*x + 2.036*x.^2);
elseif dim == 8
%     P = (8./(1-x) - 3.382.*(x - 0.16)./(x - 0.1596)).*rho;
    P = rho./(1-x.^(1./dim)) + rho.*(2.6 - 7*x);
    P = rho./(1-x.^(1./dim)) + rho.*(3.237 - 14.48*x + 21.5*x.^2);
elseif dim == 9
    P = rho.*(1./(1-x.^(1./dim)) + (3.341 - 16.83*x + 27.42*x.^2));
%     P = rho.*(1./(1-x.^(1./dim)) + (3.617 - 18.91*x + 31.29*x.^2));
elseif dim == 10
    P = rho.*(1./(1-x.^(1./dim)) + (5.503 - 33.65*x + 64.48*x.^2));
else
    P = rho./(1-(rho./cry).^(1./dim));
end

if nParticles < 1000
    if dim == 3
        if nParticles == 108
%             P = 128.3 - 266.1.*rho + 147.8.*rho.^2;
            P = (dim./(1-x) - 0.6.*(x - 0.688)./(x - 0.61)).*rho;
        elseif nParticles == 256
%             P = 124.6 - 260.5.*rho + 145.*rho.^2;
            P = (dim./(1-x) - 0.6.*(x - 0.697)./(x - 0.6)).*rho;
        else
            %nParticles = 500
%             P = 125.1 - 259.8.*rho + 144.8.*rho.^2;
            P = (dim./(1-x) - 0.6.*(x - 0.704)./(x - 0.6)).*rho;
        end
    elseif dim == 4
        if nParticles == 128
%             P = 50.98 - 77.42.*rho + 35.6.*rho.^2;
            P = (dim./(1-x) - 1.2.*(x - 0.5)./(x - 0.5)).*rho;
        else
            %nParticles = 648
%             P = 36.13 - 64.8.*rho + 36.65.*rho.^2;
            P = (dim./(1-x) - 1.07.*(x - 0.501)./(x - 0.45)).*rho;
        end
    else
        %dim = 5, %nParticles = 512
%         P = 70.72 - 91.12.*rho + 35.67.*rho.^2;
            P = (dim./(1-x) - 1.6.*(x - 0.40)./(x - 0.36)).*rho;
    end
end 
end