import monteCarloPCP as mc
import numpy as np
from scipy.special import gamma
import Resources.crystalStructures.e8Crystal as e8
import Resources.crystalStructures.e7Crystal as e7
import Resources.crystalStructures.e6Crystal as e6
import Resources.crystalStructures.dDCrystal as dD
import Resources.crystalStructures.p10cCrystal as p10c
import sys
import os
import glob
import time

# This sample script runs an einstein crystal simulation in 'dim' dimensions with number of crystal
# cells 'numCells' (see generateCrystal.py for this definition), packing fraction 'phiString', and 
# lambda (spring strength tying to reference crystal) given by 'einsteinLambda'. Runs for 'numAvgs',
# and saves with frequency 'saveFreq'. checkNeighbors is a boolean, and if cutDistances are high
# enough, it can be 0, unless d=9, in which case the internal zero mode requires that checkNeighbors
# be true (it is forced either way). If using einstein crystal as a reference (Eq. 14 in the paper),
# then isPeriodic=0, and if using the periodic reference crystal (Eq. 18 or 23), isPeriodic=1. 
# In order to place data in the folder specified in this set, then basedir = ../data/figure3/DBoundaries/
# (or ZBoundaries or EBoundaries, depending on the format of numCells).

# It is called as 
# python3 generateLiquid.py dim nParticles rhoString numR saveFreq numAvgs fullDist boundaryType basedir

dim = int(sys.argv[1])
numCells = int(sys.argv[2])
phiString = sys.argv[3]
einsteinLambdaString = sys.argv[4]
numAvgs = int(sys.argv[5])
saveFreq = int(sys.argv[6])
checkNeighbors = int(sys.argv[7])==1
isPeriodic = int(sys.argv[8])==1
basedir = sys.argv[9]

# Force checkNeighbors if using a lattice with an internal degree of freedom
if dim == 9:
	checkNeighbors = 1==1

einsteinLambda = np.float64(einsteinLambdaString)
phi = np.float64(phiString)

# Set the buffer size, how long to run before recording. This can be made higher if desired, but not lower.
numBuffer = 1000

# E boundaries are boundaryType = 1
# D boundaries are boundaryType = 2
# Z boundaries are boundaryType = 3
# volpoints give sthe number of points used when numerically integrating g(r) when r > L/2 
# (with L = box length)
boundaryType = 3
if dim==6:
	# In 6d, we must tile our box to be close to orthorhombic. numCells provides a key to which
	# option below we would like to use. Check side lengths as 'sides'
	if numCells == 19:
		numCells = np.array([3,3,3,3,3,3])
		crystalPos, sides = e6.generateE6Crystal(numCells)
	elif numCells == 20:
		numCells = np.array([2,2,2,2,2,2])
		crystalPos, sides = e6.generateE6Crystal(numCells)
	elif numCells == 21:
		numCells = np.array([2,2,2,2,2,3])
		crystalPos, sides = e6.generateE6Crystal(numCells)
	elif numCells == 22:
		numCells = np.array([2,2,2,2,3,3])
		crystalPos, sides = e6.generateE6Crystal(numCells)
	elif numCells == 23:
		numCells = np.array([2,2,2,3,3,3])
		crystalPos, sides = e6.generateE6Crystal(numCells)
	elif numCells == 24:
		crystalPos, sides = e6.generateE6Crystal(numCells,version=2)
	else:
		crystalPos, sides = e6.generateE6Crystal(numCells,version=3)
	maxPhi = e6.maxPhi()
	volPoints = 1000000
elif dim==7:
	# E7 has only one valid configuration encoded, all numCells yield the same result
	crystalPos, sides = e7.generateE7Crystal()
	maxPhi = e7.maxPhi()
	volPoints = 1000000
elif dim==8:
	# version = 1 gives E8 boundaries with N=6561
	# version = 2 gives D8 boundaries with N=13122
	# version = 3 gives Z8 boundaries with N=65536 
	crystalPos, sides = e8.generateE8Crystal(version = numCells)
	maxPhi = e8.maxPhi()
	volPoints = 1000000
	# boundary type corresponds with version number
	boundaryType = numCells
elif dim==10:
	# version = 1 gives D10^+ boundaries with N=40960
	# version = 2 gives D10 boundaries with N=81920
	crystalPos, sides = p10c.generateP10cCrystal(version = numCells)
	maxPhi = p10c.maxPhi()
	volPoints = 1000000
	# boundary type corresponds with version number
	boundaryType = numCells
else:
	# numCells > 100 gives Dn boundaries with N=(numCells - 100)^d
	# numCells < 100 gives Zn boundaries with N=0.5*(2*numCells)^d
	if numCells < 100:
		crystalPos, sides = dD.generatedDCrystal(numCells,dim)
		maxPhi = dD.maxPhi(dim)
	else:
		boundaryType = 2
		crystalPos, sides = dD.generatedDCrystalDBC(numCells-100,dim)
		maxPhi = dD.maxPhi(dim)
	volPoints = 1000000	


nParticles = crystalPos.shape[0]

saveDir = basedir + '/' + str(dim) + '/' + phiString + '/' + str(nParticles) + '/'

os.makedirs(saveDir,exist_ok=True)

# Initialize packing
p = mc.MCPacking(dim,nParticles,phi)
p.setBoxSize(sides)

# Set boundaries
if boundaryType == 3:
	boxVolume = 1
elif boundaryType==2:
	boxVolume = 2
	p.setGeometryType(mc.geometryEnum.Dn)
else: #boundaryType==1
	boxVolume = 1
	p.setGeometryType(mc.geometryEnum.E8)

maxRad = (boxVolume*maxPhi/(nParticles * p.hyperBallCoefficient(dim)))**(1/dim)
# Different definitions of maxDist which can be used.
#maxDist = 4*(maxRad - p.getRadii()[0]) + 2*p.getRadii()[0]
# maxDist = ((maxRad/p.getRadii()[0] - 1) * 4 + 1) * 2 * p.getRadii()[0]
maxDist = (maxRad - p.getRadii()[0]) * 8 + 2*p.getRadii()[0]
cutDist = maxDist/(2*p.getRadii()[0]) - 1

msd = np.zeros(numAvgs)

# Set einstein reference type
if isPeriodic:
	energy = np.zeros(numAvgs)
	const = np.zeros(numAvgs)
	if einsteinLambda < 1:
		p.setEinsteinType(mc.einsteinEnum.periodicUmbrellaExp)
	else:
		p.setEinsteinType(mc.einsteinEnum.periodic)
	if dim <= 5:
		if numCells > 100:
			latticeSpacing = 2/(numCells - 99)
		else:
			latticeSpacing = 2/numCells
	else: # dim == 8 or 9
		latticeSpacing = 2/3
	p.setLatticeSpacing(latticeSpacing)
else:
	p.setEinsteinType(mc.einsteinEnum.spring)

if dim == 8:
	cutDist = 1
elif dim >= 9:
	cutDist *= 1
elif dim == 7:
	cutDist *= 1.5
elif dim == 6:
	cutDist *= 2
else:
	cutDist *= 3

print(cutDist)
p.setPositions(crystalPos)
p.setPhi(phi)
p.calcNeighborsCut(cutDist)
minRadius = np.min(p.getRadii())
p.setEinsteinLambda(einsteinLambda)
p.setCrystalPositions(p.getUnwrappedPositions())

p.setRandomSeed(np.random.randint(2**16))
runThrough = True
startingPoint = 0

# Load previous data
if os.path.exists(saveDir + '/msd.dat'):
	msdLoad = mc.fileIO.loadArray(saveDir + '/msd.dat',dtype=np.float64)
	msdLoad = msdLoad[msdLoad!=0]
	if isPeriodic:
		energyLoad = mc.fileIO.loadArray(saveDir + '/energy.dat',dtype=np.float64)
		energyLoad = energyLoad[msdLoad!=0]
		constLoad = mc.fileIO.loadArray(saveDir + '/const.dat',dtype=np.float64)
		constLoad = constLoad[msdLoad!=0]
	if msd.size <= msdLoad.size:
		runThrough = False
	startingPoint = msdLoad.size
	for i in range(msdLoad.size):
		msd[i] = msdLoad[i]
		if isPeriodic:
			energy[i] = energyLoad[i]
			const[i] = constLoad[i]

os.makedirs(saveDir,exist_ok=True)

if runThrough:
	einsteinLambda = np.float64(einsteinLambdaString)
	print(einsteinLambda)
	p.setPositions(crystalPos)
	p.calcNeighborsCut(cutDist)
	p.calcForceEnergy()
	# Load MC max displacements
	if os.path.exists(saveDir + '/maxMCDisp.dat'):
		with open(saveDir + '/maxMCDisp.dat', 'r') as file:
			maxDisp = np.float64(file.read())
	else:
		maxDisp = p.findMCMaxDisplacement(hiDist = 0.5, distTol = 1e-3, neighborCut = cutDist,verbose = True,numTrials=10)
		with open(saveDir + '/maxMCDisp.dat', 'w') as file:
			file.write(str(maxDisp))
	p.setMCMaxDisplacement(maxDisp)
	if isPeriodic:
		if os.path.exists(saveDir + '/maxMCDispCM.dat'):
			with open(saveDir + '/maxMCDispCM.dat', 'r') as file:
				maxDispCM = np.float64(file.read())
		else:
			maxDispCM = p.findMCMaxDisplacementCM(hiDist = 10, distTol = 1e-3, verbose = True,numTrials=1000)
			with open(saveDir + '/maxMCDispCM.dat', 'w') as file:
				file.write(str(maxDispCM))
		p.setMCMaxDisplacementCM(maxDispCM)
	# The internal degree of freedom is always allowed to be activated. We find that setting it to 3 times
	# the typical move size allows rapid equilibration
	if dim == 9:
		p.setMCMaxDisplacementD9Relative(3*maxDisp)
	p.resetCenterOfMass()
	p.setPositions(crystalPos)
	p.setCrystalPositions(p.getUnwrappedPositions())
	p.calcNeighborsCut(cutDist)
	p.resetDisplacement()
	p.calcNeighborsCut(cutDist)
	# Run through a number of MC steps before we start recording
	for j in range(numBuffer):
		#Should check how far particles have moved and then recalculate neighbors if it's beyond max mc disp
		if (checkNeighbors and p.getSumOfMaxDisplacements() >= 2*minRadius*cutDist):
			# print('resetting at ' + str(j))
			print("resetting")
			p.calcNeighborsCut(cutDist)
			p.resetDisplacement()
		p.MCStep()
	print("startingPoint is " + str(startingPoint))
	for j in range(startingPoint,numAvgs):
		#Should check how far particles have moved and then recalculate neighbors if it's beyond max mc disp
		if (checkNeighbors and p.getSumOfMaxDisplacements() >= 2*minRadius*cutDist):
			print("resetting")
			p.calcNeighborsCut(cutDist)
			p.resetDisplacement()
		p.MCStep()
		# Calculate MSD if using Einstein crystal and energy/const if using periodic potential
		delta = p.getUnwrappedPositions() - p.getCenterOfMassShift() - crystalPos
		msd[j] = np.mean(np.sum((delta - np.round(delta))**2,1))
		msd[j] = np.mean(np.sum(delta**2,1))
		if isPeriodic:
			energy[j] = p.getEinsteinEnergy()/nParticles
			const[j] = p.getEinsteinWeightedConstant()
		if (j%saveFreq==0 and j>0):
			if isPeriodic:
				totalMSD = np.mean(energy[:j])/np.mean(const[:j])
			else:
				totalMSD = np.mean(msd[:j])/((2.0*np.mean(p.getRadii()))**2)
			mc.fileIO.saveArray(saveDir + '/msd.dat',msd[:j])
			if isPeriodic:
				mc.fileIO.saveArray(saveDir + '/energy.dat',energy[:j])
				mc.fileIO.saveArray(saveDir + '/const.dat',const[:j])
				totalMSD = np.mean(energy[:j])/np.mean(const[:j])
			with open(saveDir + '/msdPlateau.dat', 'w') as file:
				file.write(str(totalMSD))
			with open(saveDir + '/radii.dat','w') as file:
				file.write(str(np.mean(p.getRadii())))
	print(p.getPhi())
	mc.fileIO.saveArray(saveDir + '/msd.dat',msd)
	if isPeriodic:
		mc.fileIO.saveArray(saveDir + '/energy.dat',energy)
		mc.fileIO.saveArray(saveDir + '/const.dat',const)
	with open(saveDir + '/msdPlateau.dat', 'w') as file:
		file.write(str(totalMSD))
	with open(saveDir + '/radii.dat','w') as file:
		file.write(str(np.mean(p.getRadii())))
