#ifndef FIRE_H_
#define FIRE_H_

#include "MCPacking.h"
#include "PIMPLClasses.h"
#include "enumTypes.h"

#include <omp.h>

//Implementation
MCPacking::FIRE::FIRE(MCPacking* ptr) : packing_(ptr) {

}

/********************** FIRE **************************/
//Set the mass for each degree of freedom
void MCPacking::FIRE::setMass(double massPrefactor){
	// #pragma omp parallel for
	for(long pID = 0; pID < packing_->pi_->numParticles; pID++){
		for(int dimIndex = 0; dimIndex < packing_->pi_->nDim; dimIndex++){
			mass[pID * packing_->pi_->nDim + dimIndex] = massPrefactor/(packing_->pi_->radii[pID] * packing_->pi_->radii[pID]);
		}
	}
}

//Update position and velocity in response to an applied force
void MCPacking::FIRE::updatePositionAndVelocityFIRE(){
	// #pragma omp parallel for
	for(long pID = 0; pID < packing_->pi_->numParticles; pID++){
		double totalForce = 0.0;
		vector<double> delta(packing_->pi_->nDim);
		for(int dimIndex = 0; dimIndex < packing_->pi_->nDim; dimIndex++){
			delta[dimIndex] = velocity[pID * packing_->pi_->nDim + dimIndex] * dt + 0.5 * dt * dt * packing_->pi_->netForce[pID * packing_->pi_->nDim + dimIndex]/mass[pID * packing_->pi_->nDim + dimIndex];
			velocity[pID * packing_->pi_->nDim + dimIndex] += 0.5 * dt * packing_->pi_->netForce[pID * packing_->pi_->nDim + dimIndex]/mass[pID * packing_->pi_->nDim + dimIndex];
			totalForce += packing_->pi_->netForce[pID * packing_->pi_->nDim + dimIndex];
		}
		packing_->pi_->translateParticle(pID,delta);
		//If the total force on a particle is zero, then zero out the velocity as well
		if (totalForce == 0){
			for(int dimIndex = 0; dimIndex < packing_->pi_->nDim; dimIndex++){
				velocity[pID * packing_->pi_->nDim + dimIndex] = 0.0;
			}
		}
	}
}

//Update velocity in response to an applied force and return the maximum displacement in the previous step

void MCPacking::FIRE::updateVelocityFIRE(){
	// #pragma omp parallel for
	for(long pID = 0; pID < packing_->pi_->numParticles; pID++){
		double totalForce = 0.0;
		vector<double> delta(packing_->pi_->nDim);
		//TODO: Fix this to make it a verlet integrator
		for(int dimIndex = 0; dimIndex < packing_->pi_->nDim; dimIndex++){
			velocity[pID * packing_->pi_->nDim + dimIndex] += 0.5 * dt * packing_->pi_->netForce[pID * packing_->pi_->nDim + dimIndex]/mass[pID * packing_->pi_->nDim + dimIndex];
			totalForce += packing_->pi_->netForce[pID * packing_->pi_->nDim + dimIndex];
		}
		if (totalForce == 0.0){
			for(int dimIndex = 0; dimIndex < packing_->pi_->nDim; dimIndex++){
				velocity[pID * packing_->pi_->nDim + dimIndex] = 0.0;
			}
		}
	}
}

//Bend the velocity towards the force
void MCPacking::FIRE::bendVelocityTowardsForceFIRE(){
	// Calculate the dot product between velocity and force
	double vDotF = 0.0;
	// #pragma omp parallel for reduction(+: vDotF)
	for(long i = 0; i < packing_->pi_->numParticles * packing_->pi_->nDim; i++){
		vDotF += velocity[i] * packing_->pi_->netForce[i];
	}
	//If vDotF is negative, then we are going uphill, so let's stop and reset
	if(vDotF < 0.0){
		fill(velocity.begin(), velocity.end(), 0.0); //Zero out the velocities
		lastRestart = 0; //Reset the stopping step to now
		dt = max(dt * f_dec, dt_max/10.0); //Go to a shorter dt
		fire_alpha = a_start; //Start fresh with a more radical mixing between force and velocity
	}else if (lastRestart > N_min){
		//If enough time has passed then let's start to increase the inertia
		dt = min(dt * f_inc, dt_max);
		fire_alpha *= f_a; //Increase the inertia
	}
	double vSquared = 0.0;
	double FSquared = 0.0;
	for(long i = 0; i < packing_->pi_->numParticles * packing_->pi_->nDim; i++){
		vSquared += velocity[i] * velocity[i];
		FSquared += packing_->pi_->netForce[i] * packing_->pi_->netForce[i];
	}
	if(FSquared == 0.0){
		//If the forceNormSq is zero, then there is no force and we are done, so zero out the velocity
		fill(velocity.begin(), velocity.end(), 0.0);
	} else {
		//Now calculate the new fire velocity, using the normratio
		//d_velocity = (1-a) * d_velocity + a * d_force * normRatio
		double globalNormsRatio = sqrt(vSquared/FSquared);
		// #pragma omp parallel for
		for(long i = 0; i < packing_->pi_->numParticles * packing_->pi_->nDim; i++){
			velocity[i] = (1.0-fire_alpha) * velocity[i] + fire_alpha * packing_->pi_->netForce[i] * globalNormsRatio;
		}
	}
}

//Functions related to the minimizeFIRE
void MCPacking::FIRE::initMinimizer(double a_start_, double f_dec_, double f_inc_, double f_a_, double dt_, double dt_max_, double maxForceRescale_, long N_min_, double fire_alpha_, long lastRestart_) {
	a_start = a_start_;
	f_dec = f_dec_;
	f_inc = f_inc_;
	f_a = f_a_;
	dt = dt_;
	dt_max = dt_max_;
	N_min = N_min_;
	fire_alpha = fire_alpha_;
	lastRestart = lastRestart_;

	packing_->pi_->maxForceRescale = maxForceRescale_;

	//Create arrays to hold the force vector, velocity, mass, energy
	velocity = vector<double>(packing_->pi_->numParticles*packing_->pi_->nDim);
	//Note that mass is defined for each degree of freedom, this makes some of the FIRE stuff easier to implement
	mass = vector<double>(packing_->pi_->numParticles*packing_->pi_->nDim);
	//Set the velocity, force, and energy to zero
	std::fill(velocity.begin(), velocity.end(), 0.0);
	//Set the mass for each degree of freedom
	//TODO: Can we eliminate the mass term and just use a dynamic calculation based on radii (or set them all to 1)?
	setMass(1.0/packing_->pi_->calcVolumePrefactor(packing_->pi_->nDim));
	//Create energy and force arrays
	packing_->initForceEnergy();

	packing_->calcForceEnergy();
	if (packing_->pi_->simControl.potentialType == simControlStruct::potentialEnum::log || packing_->pi_->simControl.potentialType == simControlStruct::potentialEnum::wca || packing_->pi_->simControl.potentialType == simControlStruct::potentialEnum::inversePower) {
		packing_->pi_->rescaleForces(packing_->pi_->maxForceRescale);
	}
}

void MCPacking::FIRE::minimizerLoop(){
	//TODO: Deal with the neccessity of scaling the log forces here, perhaps by reordering?
	updatePositionAndVelocityFIRE(); //Move the system forward, based on the previous velocities and forces
	packing_->calcForceEnergy(); //Calculate the new set of forces at the new step
	// cout << __PRETTY_FUNCTION__ << " Energy = " << string(hostnum(thrust::reduce(thrust::cuda::par(packing_->pi_->alloc), packing_->pi_->d_energy.begin(),packing_->pi_->d_energy.end(), devnum(0), thrust::plus<devnum>() ) )) << endl;
	if (packing_->pi_->simControl.potentialType == simControlStruct::potentialEnum::log || packing_->pi_->simControl.potentialType == simControlStruct::potentialEnum::wca || packing_->pi_->simControl.potentialType == simControlStruct::potentialEnum::inversePower) {
		packing_->pi_->rescaleForces(packing_->pi_->maxForceRescale);
	}
	updateVelocityFIRE(); //update the velocity, based on the current forces
	bendVelocityTowardsForceFIRE(); //Bend the velocity towards the force
	if (packing_->pi_->simControl.constraintType == simControlStruct::constraintEnum::strainControl){
		//If doing constrained minimization, velocity will never point in the direction of the constraint
		//because force doesn't, but it is safest to project it out to prevent noise leaking. 
		packing_->pi_->projectVector(packing_->fire_->velocity);
	}
	lastRestart++; //Increase the number of steps since the last restart
}


#endif /* FIRE_H_ */